/*{
    "CATEGORIES": [
        "Distortion Effect"
    ],
    "CREDIT": "VIDVOX // Updated by ProjectileObjects",
    "DESCRIPTION": "Version 3 Warps an image to fit in a circle by fitting the height of the image to the height of a circle (updated with Median Noise filtering, alpha, height, and color options)",
    "INPUTS": [
        {
            "NAME": "inputImage",
            "TYPE": "image"
        },
        {
            "DEFAULT": 0.5,
            "MAX": 0.5,
            "MIN": 0,
            "NAME": "radius",
            "TYPE": "float"
        },
        {
            "DEFAULT": 1,
            "MAX": 2,
            "MIN": 0,
            "NAME": "width",
            "TYPE": "float"
        },
        {
            "DEFAULT": 0,
            "MAX": 1,
            "MIN": 0,
            "NAME": "resultRotation",
            "TYPE": "float"
        },
        {
            "DEFAULT": 0,
            "NAME": "medNoise",
            "TYPE": "float"
        },
        {
            "DEFAULT": 1,
            "NAME": "height",
            "TYPE": "float"
        },
        {
            "NAME": "colorR",
            "TYPE": "float"
        },
        {
            "NAME": "colorG",
            "TYPE": "float"
        },
        {
            "NAME": "colorB",
            "TYPE": "float"
        },
        {
            "DEFAULT": 0,
            "NAME": "colorA",
            "TYPE": "float"
        }
    ],
    "ISFVSN": "2"
}
*/
vec4 median(vec2 center, float radius) {
    vec4 colorSamples[9];
    int i = 0;

    for (int y = -1; y <= 1; y++) {
        for (int x = -1; x <= 1; x++) {
            vec2 sampleCoord = center + vec2(float(x), float(y)) * radius;
            colorSamples[i] = IMG_NORM_PIXEL(inputImage, sampleCoord);
            i++;
        }
    }

    // Sort the colors based on luminance (you can use a different metric)
    for (i = 0; i < 9; i++) {
        for (int j = i + 1; j < 9; j++) {
            if (dot(colorSamples[i].rgb, vec3(0.299, 0.587, 0.114)) > dot(colorSamples[j].rgb, vec3(0.299, 0.587, 0.114))) {
                vec4 temp = colorSamples[i];
                colorSamples[i] = colorSamples[j];
                colorSamples[j] = temp;
            }
        }
    }

    // Return the median color
    return colorSamples[4];
}

const float pi = 3.1415926535897932384626433832795;

vec2 rotatePointNorm(vec2 pt, float rot)    {
    vec2 returnMe = pt;

    float r = distance(vec2(0.50), returnMe);
    float a = atan((returnMe.y-0.5),(returnMe.x-0.5));

    returnMe.x = r * cos(a + 2.0 * pi * rot - pi) + 0.5;
    returnMe.y = r * sin(a + 2.0 * pi * rot - pi) + 0.5;
    
    returnMe = returnMe;
    
    return returnMe;
}

void main() {
    vec4        inputPixelColor;
    vec2        pt = isf_FragNormCoord;
    vec2        ct = vec2(0.5,0.5);
    
    pt -= ct;
    pt.x /= width;
    pt.y /= height;
    pt += ct;
    
    pt = mix(vec2((pt.x*RENDERSIZE.x/RENDERSIZE.y)-(RENDERSIZE.x*.5-RENDERSIZE.y*.5)/RENDERSIZE.y,pt.y), 
                vec2(pt.x,pt.y*(RENDERSIZE.y/RENDERSIZE.x)-(RENDERSIZE.y*.5-RENDERSIZE.x*.5)/RENDERSIZE.x), 
                step(RENDERSIZE.x,RENDERSIZE.y));
    pt = rotatePointNorm(pt,resultRotation+0.5);
    
    if (distance(pt,ct) < radius)   {
     vec4 sampleColor = IMG_NORM_PIXEL(inputImage, pt);
     // Apply Gaussian blur to reduce noise
        float blurRadius = 0.5; // Adjust the radius as needed
        inputPixelColor = IMG_NORM_PIXEL(inputImage, pt);
        for (int i = -2; i <= 2; i++) {
            for (int j = -2; j <= 2; j++) {
                inputPixelColor += IMG_NORM_PIXEL(inputImage, pt + vec2(float(i), float(j)) * blurRadius);
            }
        }
        inputPixelColor /= 25.0; // Adjust the divisor based on the blur kernel size
        float       a = (atan(ct.y-pt.y,ct.x-pt.x) + pi) / (2.0*pi);
        //inputPixelColor = IMG_NORM_PIXEL(inputImage,pt);
        pt.y -= 0.5;
        pt.y /= 2.0*sqrt(pow(radius,2.0)-pow((pt.x-0.5),2.0));
        pt.y += 0.5;
        inputPixelColor = IMG_NORM_PIXEL(inputImage,pt);
        // Apply median filtering to remove noise
        inputPixelColor = median(pt, medNoise); // Adjust the radius as needed

        // Optionally, you can apply additional operations or filters here
    } else {
        // If not in the background noise region, set alpha to 0 (transparent)
        inputPixelColor = vec4(colorR, colorG, colorB, colorA);
    }


    gl_FragColor = inputPixelColor;
}